﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Reflection;
using System.Xml;
using System.Windows.Markup;

namespace DumpControlTemplate
{
    public partial class MainWindow : Window
    {
        public Control _Control { get; set; }

        public MainWindow()
        {
            InitializeComponent();
        }

        private void ControlItemOnClick(object sender, RoutedEventArgs e)
        {
            for (int i = 0, j = grd.Children.Count; i < j; i++)
            {
                if (Grid.GetRow(grd.Children[i]) == 0)
                {
                    grd.Children.Remove(grd.Children[i]);
                    break;
                }
            }
            txtBox.Text = string.Empty;
            var item = e.Source as MenuItem;
            var type = ((Type)item.Tag);
            ConstructorInfo info = type.GetConstructor(Type.EmptyTypes);
            try
            {
                _Control = ((Control)info.Invoke(null));
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, Title);
                return;
            }

            try
            {
                grd.Children.Add(_Control);
            }
            catch
            {
                if (_Control is Window)
                    (_Control as Window).Show();
                else return;
            }
            Title = Title.Remove(Title.IndexOf('-')) + "- " + type.Name;
        }

        private void DumpOnOpened(object sender, RoutedEventArgs e)
        {
            ItemTemplate.IsEnabled = _Control != null;
            ItemItemsPanel.IsEnabled = _Control != null && _Control is ItemsControl;
        }

        private void DumpItemsPanelOnClick(object sender, RoutedEventArgs e)
        {
            if (_Control != null && _Control is ItemsControl)
                Dump((_Control as ItemsControl).ItemsPanel);
        }

        private void Dump(FrameworkTemplate template)
        {
            if (template != null)
            {
                var settings = new XmlWriterSettings
                {
                    Indent = true,
                    IndentChars = new string(' ', 4),
                    NewLineOnAttributes = true
                };

                var strbuild = new StringBuilder();
                var xmlWrite = XmlWriter.Create(strbuild, settings);
                try
                {
                    XamlWriter.Save(template, xmlWrite);
                    txtBox.Text = strbuild.ToString();
                }
                catch (Exception ex)
                {
                    txtBox.Text = ex.Message;
                }
            }
            else
            {
                txtBox.Text = "no Template";
            }
        }

        private void DumpTemplateOnClick(object sender, RoutedEventArgs e)
        {
            if (_Control != null)
                Dump(_Control.Template);
        }
    }

    public class ControlMenuItem : MenuItem
    {
        public ControlMenuItem()
        {
            var assembly = Assembly.GetAssembly(typeof(Control));
            var aTypes = assembly.GetTypes();
            var sortList = new SortedList<string, MenuItem>();
            Header = "Control";
            Tag = typeof(Control);
            sortList.Add("Control", this);
            foreach (var type in aTypes)
            {
                if (type.IsPublic &&
                    type.IsSubclassOf(typeof(Control)))
                {
                    sortList.Add(type.Name,
                        new MenuItem
                        {
                            Header = type.Name,
                            Tag = type
                        });
                }
            }

            foreach (var kvp in sortList)
            {
                if (kvp.Key == "Control") continue;
                var strParent = ((Type)kvp.Value.Tag).BaseType.Name;
                var itemParent = sortList[strParent];
                itemParent.Items.Add(kvp.Value);
            }

            foreach (var kvp in sortList)
            {
                var type = (Type)kvp.Value.Tag;
                if (type.IsAbstract && kvp.Value.Items.Count == 0)
                    kvp.Value.IsEnabled = false;
                else
                    kvp.Value.Items.Insert(0,
                        new MenuItem
                        {
                            Header = kvp.Value.Header as string,
                            Tag = type
                        });
            }
        }
    }
}
